// Package pad writes padding to memory buffers.
//
// Although the functions take an io.Writer, the writers must be memory
// buffers and never return an error.
package pad

import "io"

// WritePadding writes count bytes from the fill, repeating fill as necessary.
func WritePadding(w io.Writer, count int, fill string) {
	fl := len(fill)
	for fl < count {
		_, _ = io.WriteString(w, fill)
		count -= fl
	}

	_, _ = io.WriteString(w, fill[:count])
}

// WriteSpaces writes count spaces.
func WriteSpaces(w io.Writer, count int) {
	WritePadding(w, count, "                                ")
}
